@@ -1,4 +1,6 @@
-/* Copyright (c) 1992, Free Software Foundation, Inc.  All rights reserved.
+/* Copyright (c) 1994, Joseph Arceneaux.  All rights reserved
+
+   Copyright (c) 1992, Free Software Foundation, Inc.  All rights reserved.
 
    Copyright (c) 1985 Sun Microsystems, Inc. Copyright (c) 1980 The Regents
    of the University of California. Copyright (c) 1976 Board of Trustees of
@@ -26,6 +28,12 @@
 #include "indent.h"
 #include <ctype.h>
 
+#if defined (HAVE_UNISTD_H)
+#include <unistd.h>
+#endif
+#include <string.h>
+
+
 /* Stuff that needs to be shared with the rest of indent. Documented in
    indent.h.  */
 char *token;
@@ -71,96 +79,108 @@ char chartype[128] =
   1, 1, 1, 0, 3, 0, 3, 0
 };
 
-/* C code produced by gperf version 2.0 (K&R C version)
-   Command-line:
-   gperf -c -p -t -T -g -j1 -o -K rwd -N is_reserved indent.gperf  */
+/* C code produced by gperf version 2.5 (GNU C++ version) */
+/* Command-line: gperf -D -c -p -t -T -g -j1 -o -K rwd -N is_reserved indent-c++.gperf  */
 
+#define TOTAL_KEYWORDS 39
 #define MIN_WORD_LENGTH 2
-#define MAX_WORD_LENGTH 8
+#define MAX_WORD_LENGTH 9
 #define MIN_HASH_VALUE 4
-#define MAX_HASH_VALUE 42
+#define MAX_HASH_VALUE 67
+/* maximum key range = 64, duplicates = 0 */
 
-/* 31 keywords. 39 is the maximum key range */
-
-INLINE
-static int
-hash (str, len)
-     register char *str;
-     register unsigned int  len;
+#ifdef __GNUC__
+inline
+#endif
+static unsigned int
+hash (char *str, int unsigned len)
 {
-  static unsigned char hash_table[] =
+  static unsigned char asso_values[] =
     {
-     42, 42, 42, 42, 42, 42, 42, 42, 42, 42,
-     42, 42, 42, 42, 42, 42, 42, 42, 42, 42,
-     42, 42, 42, 42, 42, 42, 42, 42, 42, 42,
-     42, 42, 42, 42, 42, 42, 42, 42, 42, 42,
-     42, 42, 42, 42, 42, 42, 42, 42, 42, 42,
-     42, 42, 42, 42, 42, 42, 42, 42, 42, 42,
-     42, 42, 42, 42, 42, 42, 42, 42, 42, 42,
-     42, 42, 42, 42, 42, 42, 42, 42, 42, 42,
-     42, 42, 42, 42, 42, 42, 42, 42, 42, 42,
-     42, 42, 42, 42, 42, 42, 42, 42,  6,  9,
-     10,  0, 16,  5,  4, 24, 42,  0, 20,  4,
-     20,  0, 42, 42,  6,  0,  0, 10, 10,  2,
-     42, 42, 42, 42, 42, 42, 42, 42,
+     68, 68, 68, 68, 68, 68, 68, 68, 68, 68,
+     68, 68, 68, 68, 68, 68, 68, 68, 68, 68,
+     68, 68, 68, 68, 68, 68, 68, 68, 68, 68,
+     68, 68, 68, 68, 68, 68, 68, 68, 68, 68,
+     68, 68, 68, 68, 68, 68, 68, 68, 68, 68,
+     68, 68, 68, 68, 68, 68, 68, 68, 68, 68,
+     68, 68, 68, 68, 68, 68, 68, 68, 68, 68,
+     68, 68, 68, 68, 68, 68, 68, 68, 68, 68,
+     68, 68, 68, 68, 68, 68, 68, 68, 68, 68,
+     68, 68, 68, 68, 68, 68, 68, 68, 31,  4,
+      0,  0, 21, 33,  6, 19, 68,  0, 20,  1,
+     23, 30, 14, 68,  6,  7,  0, 10,  7,  5,
+     68, 68, 68, 68, 68, 68, 68, 68,
     };
-  return len + hash_table[str[len - 1]] + hash_table[str[0]];
+
+  return len + asso_values[(int) str[len - 1]] + asso_values[(int) str[0]];
 }
 
-
-INLINE
+#ifdef __GNUC__
+inline
+#endif
 struct templ*
 is_reserved (str, len)
-     register char *str;
-     register unsigned int len;
+     char *str;
+     unsigned int len;
 {
-
   static struct templ wordlist[] =
     {
       {"",}, {"",}, {"",}, {"",}, 
       {"else",  rw_sp_nparen,},
-      {"short",  rw_decl,},
-      {"struct",  rw_struct_like,},
-      {"while",  rw_sp_paren,},
       {"enum",  rw_struct_like,},
-      {"goto",  rw_break,},
-      {"switch",  rw_switch,},
-      {"break",  rw_break,},
-      {"do",  rw_sp_nparen,},
-      {"case",  rw_case,},
-      {"const",  rw_decl,},
-      {"static",  rw_decl,},
       {"double",  rw_decl,},
       {"default",  rw_case,},
-      {"volatile",  rw_decl,},
-      {"char",  rw_decl,},
-      {"register",  rw_decl,},
-      {"float",  rw_decl,},
-      {"sizeof",  rw_sizeof,},
-      {"typedef",  rw_decl,},
+      {"case",  rw_case,},
+      {"const",  rw_decl,},
+      {"while",  rw_sp_paren,},
       {"void",  rw_decl,},
-      {"for",  rw_sp_paren,},
-      {"extern",  rw_decl,},
-      {"int",  rw_decl,},
+      {"short",  rw_decl,},
+      {"struct",  rw_struct_like,},
+      {"char",  rw_decl,},
+      {"volatile",  rw_decl,},
+      {"class",  rw_struct_like,},
+      {"static",  rw_decl,},
       {"unsigned",  rw_decl,},
+      {"switch",  rw_switch,},
+      {"register",  rw_decl,},
+      {"private",  rw_decl,},
+      {"int",  rw_decl,},
+      {"protected",  rw_decl,},
+      {"public",  rw_decl,},
+      {"inline",  rw_decl,},
+      {"float",  rw_decl,},
+      {"friend",  rw_decl,},
+      {"typedef",  rw_decl,},
+      {"extern",  rw_decl,},
+      {"for",  rw_sp_paren,},
+      {"new",  rw_sizeof,},
+      {"do",  rw_sp_nparen,},
+      {"va_dcl",  rw_decl,},
+      {"sizeof",  rw_sizeof,},
+      {"return",  rw_return,},
+      {"break",  rw_break,},
+      {"",}, 
+      {"union",  rw_struct_like,},
+      {"",}, {"",}, {"",}, 
+      {"if",  rw_sp_paren,},
+      {"",}, 
+      {"operator",  rw_operator,},
+      {"",}, {"",}, {"",}, {"",}, {"",}, {"",}, {"",}, {"",}, {"",}, 
+      {"",}, {"",}, {"",}, 
       {"long",  rw_decl,},
       {"",}, 
       {"global",  rw_decl,},
-      {"return",  rw_return,},
-      {"",}, {"",}, 
-      {"union",  rw_struct_like,},
-      {"va_dcl",  rw_decl,},
-      {"",}, {"",}, {"",}, {"",}, {"",}, 
-      {"if",  rw_sp_paren,},
+      {"",}, {"",}, {"",}, {"",}, {"",}, {"",}, {"",}, 
+      {"goto",  rw_break,},
     };
 
   if (len <= MAX_WORD_LENGTH && len >= MIN_WORD_LENGTH)
     {
-      register int key = hash (str, len);
+      int key = hash (str, len);
 
-      if (key <= MAX_HASH_VALUE && key >= MIN_HASH_VALUE)
+      if (key <= MAX_HASH_VALUE && key >= 0)
         {
-          register char *s = wordlist[key].rwd;
+          char *s = wordlist[key].rwd;
 
           if (*s == *str && !strncmp (str + 1, s + 1, len - 1))
             return &wordlist[key];
@@ -169,6 +189,7 @@ is_reserved (str, len)
   return 0;
 }
 
+
 extern int squest;
 
 enum codes
@@ -181,28 +202,37 @@ lexi ()
   enum codes code;		/* internal code to be returned */
   char qchar;			/* the delimiter character for a string */
 
+  static int count;		/* debugging counter */
+  count++;
+
   unary_delim = false;
   /* tell world that this token started in column 1 iff the last
      thing scanned was nl */
   parser_state_tos->col_1 = parser_state_tos->last_nl;
   parser_state_tos->last_nl = false;
 
-  while (*buf_ptr == ' ' || *buf_ptr == TAB)
-    {				/* get rid of blanks */
-      parser_state_tos->col_1 = false;	/* leading blanks imply token is not
-					   in column 1 */
-      if (++buf_ptr >= buf_end)
-	fill_buffer ();
+  if  (*buf_ptr == ' ' || *buf_ptr == TAB)
+    {
+      parser_state_tos->col_1 = false;
+      while (*buf_ptr == ' ' || *buf_ptr == TAB)
+	if (++buf_ptr >= buf_end)
+	  fill_buffer ();
     }
 
+  /* INCREDIBLY IMPORTANT WARNING!!!
+
+     Note that subsequent calls to `fill_buffer ()' may switch `buf_ptr'
+     to a different buffer.  Thus when `token_end' gets set later, it
+     may be pointing into a different buffer than `token'. */
   token = buf_ptr;
 
   /* Scan an alphanumeric token */
-  if (chartype[*buf_ptr] == alphanum
+  if ((! (buf_ptr[0] == 'L' && (buf_ptr[1] == '"' || buf_ptr[1] == '\''))
+          && chartype[(int) *buf_ptr] == alphanum)
       || (buf_ptr[0] == '.' && isdigit (buf_ptr[1])))
     {
       /* we have a character or number */
-      register struct templ *p;
+      struct templ *p;
 
       if (isdigit (*buf_ptr) || (buf_ptr[0] == '.' && isdigit (buf_ptr[1])))
 	{
@@ -212,34 +242,38 @@ lexi ()
 	    {
 	      buf_ptr += 2;
 	      while (isxdigit (*buf_ptr))
-		buf_ptr++;
+		{
+		  buf_ptr++;
+		}
 	    }
 	  else
 	    while (1)
 	      {
 		if (*buf_ptr == '.')
-		  if (seendot)
-		    break;
-		  else
-		    seendot++;
+		  {
+		    if (seendot)
+		      break;
+		    else
+		      seendot++;
+		  }
 		buf_ptr++;
 		if (!isdigit (*buf_ptr) && *buf_ptr != '.')
-		  if ((*buf_ptr != 'E' && *buf_ptr != 'e') || seenexp)
-		    break;
-		  else
-		    {
-		      seenexp++;
-		      seendot++;
-		      buf_ptr++;
-		      if (*buf_ptr == '+' || *buf_ptr == '-')
+		  {
+		    if ((*buf_ptr != 'E' && *buf_ptr != 'e') || seenexp)
+		      break;
+		    else
+		      {
+			seenexp++;
+			seendot++;
 			buf_ptr++;
-		    }
+			if (*buf_ptr == '+' || *buf_ptr == '-')
+			  buf_ptr++;
+		      }
+		  }
 	      }
-	  /* Accept unsigned, unsigned long, and float constants
-	     (U, UL, and F suffixes).  I'm not sure if LU is ansii.
-	     11Jun93 - rdh@key.amdahl.com tells me LL and ULL are also
-	     acceptable. */
-	  if (*buf_ptr == 'F' || *buf_ptr == 'f')
+
+	  if (*buf_ptr == 'F' || *buf_ptr == 'f'
+	      || *buf_ptr == 'i' || *buf_ptr == 'j')
 	    buf_ptr++;
 	  else
 	    {
@@ -252,13 +286,15 @@ lexi ()
 	    }
 	}
       else
-	while (chartype[*buf_ptr] == alphanum)
+	while (chartype[(int) *buf_ptr] == alphanum)
 	  {			/* copy it over */
 	    buf_ptr++;
 	    if (buf_ptr >= buf_end)
 	      fill_buffer ();
 	  }
+
       token_end = buf_ptr;
+
       while (*buf_ptr == ' ' || *buf_ptr == TAB)
 	{
 	  if (++buf_ptr >= buf_end)
@@ -274,6 +310,8 @@ lexi ()
 	  l_struct = false;
 	  last_code = ident;
 	  parser_state_tos->last_u_d = true;
+	  if (parser_state_tos->last_token == cpp_operator)
+	    return overloaded;
 	  return (decl);
 	}
 
@@ -320,16 +358,26 @@ lexi ()
 
       if (p)
 	{			/* we have a keyword */
+	  enum codes value;
+
 	found_keyword:
+	  value = ident;
 	  parser_state_tos->its_a_keyword = true;
 	  parser_state_tos->last_u_d = true;
 	  parser_state_tos->last_rw = p->rwcode;
+	  parser_state_tos->last_rw_depth = parser_state_tos->paren_depth;
 	  switch (p->rwcode)
 	    {
+	    case rw_operator:	/* C++ operator overloading. */
+	      value = cpp_operator;
+	      parser_state_tos->in_parameter_declaration = 1;
+	      break;
 	    case rw_switch:	/* it is a switch */
-	      return (swstmt);
+	      value = (swstmt);
+	      break;
 	    case rw_case:	/* a case or default */
-	      return (casestmt);
+	      value = (casestmt);
+	      break;
 
 	    case rw_struct_like:	/* a "struct" */
 	      if (parser_state_tos->p_l_follow
@@ -356,94 +404,117 @@ lexi ()
 		  break;
 		}
 	      last_code = decl;
-	      return (decl);
+	      value = (decl);
+	      break;
 
 	    case rw_sp_paren:	/* if, while, for */
-	      return (sp_paren);
+	      value = (sp_paren);
+	      break;
 
 	    case rw_sp_nparen:	/* do, else */
-	      return (sp_nparen);
+	      value = (sp_nparen);
+	      break;
 
 	    case rw_sizeof:
 	      parser_state_tos->sizeof_keyword = true;
-	      return (ident);
+	      value = (ident);
+	      break;
 
 	    case rw_return:
 	    case rw_break:
 	    default:		/* all others are treated like any other
 				   identifier */
-	      return (ident);
+	      value = (ident);
 	    }			/* end of switch */
+
+	  if (parser_state_tos->last_token == cpp_operator)
+	    return overloaded;
+	  return value;
 	}			/* end of if (found_it) */
+      else if (*buf_ptr == '('
+	       && parser_state_tos->tos <= 1
+	       && parser_state_tos->ind_level == 0
+	       && parser_state_tos->paren_depth == 0)
+	  {
+	    /* We have found something which might be the name in a function
+	       definition.  */
+	    char *tp;
+	    int paren_count = 1;
 
-      if (*buf_ptr == '('
-	  && parser_state_tos->tos <= 1
-	  && parser_state_tos->ind_level == 0
-	  && parser_state_tos->paren_depth == 0)
-	{
-	  /* We have found something which might be the name in a function
-	     definition.  */
-	  register char *tp;
-	  int paren_count = 1;
+	    /* Skip to the matching ')'.  */
+	    for (tp = buf_ptr + 1;
+		 paren_count > 0 && tp < in_prog + in_prog_size;
+		 tp++)
+	      {
+		if (*tp == '(')
+		  paren_count++;
+		if (*tp == ')')
+		  paren_count--;
+		/* Can't occur in parameter list; this way we don't search the
+		   whole file in the case of unbalanced parens.  */
+		if (*tp == ';')
+		  goto not_proc;
+	      }
 
-	  /* Skip to the matching ')'.  */
-	  for (tp = buf_ptr + 1;
-	       paren_count > 0 && tp < in_prog + in_prog_size;
-	       tp++)
-	    {
-	      if (*tp == '(')
-		paren_count++;
-	      if (*tp == ')')
-		paren_count--;
-	      /* Can't occur in parameter list; this way we don't search the
-	         whole file in the case of unbalanced parens.  */
-	      if (*tp == ';')
-		goto not_proc;
-	    }
+	    if (paren_count == 0)
+	      {
+		parser_state_tos->procname = token;
+		parser_state_tos->procname_end = token_end;
 
-	  if (paren_count == 0)
-	    {
-	      while (isspace (*tp))
-		tp++;
-	      /* If the next char is ';' or ',' or '(' we have a function
-	         declaration, not a definition.
+		while (isspace (*tp))
+		  tp++;
 
-		 I've added '=' to this list to keep from breaking
-		 a non-valid C macro from libc.  -jla */
-	      if (*tp != ';' && *tp != ',' && *tp != '(' && *tp != '=')
-		{
-		  parser_state_tos->procname = token;
-		  parser_state_tos->procname_end = token_end;
-		  parser_state_tos->in_parameter_declaration = 1;
-		}
-	    }
-
-	not_proc:;
-	}
+		/* If the next char is ';' or ',' or '(' we have a function
+		   declaration, not a definition.
+		   
+		   I've added '=' to this list to keep from breaking
+		   a non-valid C macro from libc.  -jla */
+		if (*tp != ';' && *tp != ',' && *tp != '(' && *tp != '=')
+		  {
+		    parser_state_tos->in_parameter_declaration = 1;
+		  }
+	      }
 
+	  not_proc:;
+	  }
+      else if (*buf_ptr == ':' && *(buf_ptr + 1) == ':'
+	       && parser_state_tos->tos <= 1
+	       && parser_state_tos->ind_level == 0
+	       && parser_state_tos->paren_depth == 0)
+	  {
+	    parser_state_tos->classname = token;
+	    parser_state_tos->classname_end = token_end;
+	  }
       /* The following hack attempts to guess whether or not the
 	 current token is in fact a declaration keyword -- one that
 	 has been typedef'd */
-      if (((*buf_ptr == '*' && buf_ptr[1] != '=')
-	   || isalpha (*buf_ptr) || *buf_ptr == '_')
-	  && !parser_state_tos->p_l_follow
-	  && !parser_state_tos->block_init
-	  && (parser_state_tos->last_token == rparen
-	      || parser_state_tos->last_token == semicolon
-	      || parser_state_tos->last_token == decl
-	      || parser_state_tos->last_token == lbrace
-	      || parser_state_tos->last_token == rbrace))
-	{
-	  parser_state_tos->its_a_keyword = true;
-	  parser_state_tos->last_u_d = true;
-	  last_code = decl;
-	  return decl;
-	}
+      else if (((*buf_ptr == '*' && buf_ptr[1] != '=')
+		|| isalpha (*buf_ptr) || *buf_ptr == '_')
+	       && !parser_state_tos->p_l_follow
+	       && !parser_state_tos->block_init
+	       && (parser_state_tos->last_token == rparen
+		   || parser_state_tos->last_token == semicolon
+		   || parser_state_tos->last_token == rbrace
+		   || parser_state_tos->last_token == decl
+		   || parser_state_tos->last_token == lbrace
+		   || parser_state_tos->last_token == start_token))
+	  {
+	      parser_state_tos->its_a_keyword = true;
+	      parser_state_tos->last_u_d = true;
+	      last_code = decl;
+	      if (parser_state_tos->last_token == cpp_operator)
+		  return overloaded;
+
+	      return decl;
+	  }
 
       if (last_code == decl)	/* if this is a declared variable, then
 				   following sign is unary */
 	parser_state_tos->last_u_d = true;	/* will make "int a -1" work */
       last_code = ident;
+      if (parser_state_tos->last_token == cpp_operator)
+	return overloaded;
+
       return (ident);		/* the ident is not in the list */
     }				/* end of procesing for alpanum character */
   /* Scan a non-alphanumeric token */
@@ -451,6 +522,11 @@ lexi ()
   /* If it is not a one character token, token_end will get changed later.  */
   token_end = buf_ptr + 1;
 
+  /* THIS MAY KILL YOU!!!
+
+     Note that it may be possible for this to kill us--if `fill_buffer'
+     at any time switches `buf_ptr' to the other input buffer, `token'
+     and `token_end' will point to different storage areas!!! */
   if (++buf_ptr >= buf_end)
     fill_buffer ();
 
@@ -466,10 +542,24 @@ lexi ()
       code = newline;
       break;
 
+    /* Handle wide strings and chars. */
+    case 'L':
+      if (buf_ptr[0] != '"' && buf_ptr[0] != '\'')
+	{
+	  token_end = buf_ptr;
+	  code = ident;
+	  break;
+	}
+
+      qchar = buf_ptr[0];
+      buf_ptr++;
+      goto handle_string;
+
     case '\'':			/* start of quoted character */
     case '"':			/* start of string */
       qchar = *token;
 
+    handle_string:
       /* Find out how big the literal is so we can set token_end.  */
 
       /* Invariant:  before loop test buf_ptr points to the next */
@@ -490,12 +580,13 @@ lexi ()
 	  if (buf_ptr >= buf_end)
 	    fill_buffer ();
 	}
+
       if (*buf_ptr == EOL || *buf_ptr == 0)
 	{
-	  diag (1, (qchar == '\''
+	  WARNING ((qchar == '\''
 		    ? "Unterminated character constant"
 		    : "Unterminated string constant"),
-		0, 0);
+		   0, 0);
 	}
       else
 	{
@@ -505,6 +596,7 @@ lexi ()
 	    fill_buffer ();
 	}
 
+      token_end = buf_ptr;
       code = ident;
       break;
 
@@ -522,6 +614,15 @@ lexi ()
     case '#':
       unary_delim = parser_state_tos->last_u_d;
       code = preesc;
+
+      /* Make spaces between '#' and the directive be part of
+	 the token if user specified "-lps" */
+      if (leave_preproc_space)
+	{
+	  while (*buf_ptr == ' ' && buf_ptr < buf_end)
+	    buf_ptr++;
+	  token_end = buf_ptr;
+	}
       break;
 
     case '?':
@@ -530,6 +631,15 @@ lexi ()
       break;
 
     case (':'):
+      /* Deal with C++ class::method */
+      if (*buf_ptr == ':')
+	{
+	  code = doublecolon;
+	  buf_ptr++;
+	  token_end = buf_ptr;
+	  break;
+	}
+
       code = colon;
       unary_delim = true;
       if (squest && *e_com != ' ')
@@ -586,7 +696,7 @@ lexi ()
 
     case '.':
       unary_delim = false;
-      code = period;
+      code = struct_delim;
       break;
 
     case '-':
@@ -613,18 +723,16 @@ lexi ()
 	{
 	  /* check for operator -> */
 	  buf_ptr++;
-	  if (!pointer_as_binop)
-	    {
-	      unary_delim = false;
-	      code = unary_op;
-	      parser_state_tos->want_blank = false;
-	    }
+	  code = struct_delim;
 	}
+
+      token_end = buf_ptr;
       break;			/* buffer overflow will be checked at end of
 				   switch */
 
     case '=':
-      if (parser_state_tos->in_or_st)
+      if (parser_state_tos->in_or_st &&
+	  parser_state_tos->last_token != cpp_operator)
 	parser_state_tos->block_init = 1;
 
       if (*buf_ptr == '=')	/* == */
@@ -641,13 +749,13 @@ lexi ()
 	     people want to detect and eliminate old style assignments but
 	     they don't want indent to silently change the meaning of their
 	     code).  */
-	  diag (0,
-	  "old style assignment ambiguity in \"=%c\".  Assuming \"= %c\"\n",
-		(int) *buf_ptr, (int) *buf_ptr);
+	  WARNING ("old style assignment ambiguity in \"=%c\".  Assuming \"= %c\"\n",
+		   (int) *buf_ptr, (int) *buf_ptr);
 	}
 
       code = binary_op;
       unary_delim = true;
+      token_end = buf_ptr;
       break;
       /* can drop thru!!! */
 
@@ -670,8 +778,22 @@ lexi ()
 
       code = (parser_state_tos->last_u_d ? unary_op : binary_op);
       unary_delim = true;
+      token_end = buf_ptr;
       break;
 
+#if 0
+      /* Contributed by Beverly Brown, <beverly@norton.datacube.com> */
+    case '*':
+      /* If the * is part of a declaration such as char *,
+	 we're still a decl */
+      if(parser_state_tos->last_token == decl)
+	{
+	  unary_delim = parser_state_tos->last_u_d;
+	  last_code = decl;
+	  return decl;
+	}
+#endif
+
     default:
       if (token[0] == '/' && (*buf_ptr == '*' || *buf_ptr == '/'))
 	{
@@ -686,18 +808,26 @@ lexi ()
 	    fill_buffer ();
 
 	  unary_delim = parser_state_tos->last_u_d;
-	  break;
 	}
-
-      while (*(buf_ptr - 1) == *buf_ptr || *buf_ptr == '=')
+      else if (parser_state_tos->last_token == cpp_operator)
 	{
-	  /* handle ||, &&, etc, and also things as in int *****i */
-	  if (++buf_ptr >= buf_end)
-	    fill_buffer ();
+	  /* For C++ overloaded operators. */
+	  code = overloaded;
+	  last_code = overloaded;
+	}
+      else
+	{
+	  while (*(buf_ptr - 1) == *buf_ptr || *buf_ptr == '=')
+	    {
+	      /* handle ||, &&, etc, and also things as in int *****i */
+	      if (++buf_ptr >= buf_end)
+		fill_buffer ();
+	    }
+	  code = (parser_state_tos->last_u_d ? unary_op : binary_op);
+	  unary_delim = true;
 	}
-      code = (parser_state_tos->last_u_d ? unary_op : binary_op);
-      unary_delim = true;
 
+      token_end = buf_ptr;
 
     }				/* end of switch */
 
@@ -706,20 +836,23 @@ lexi ()
       l_struct = false;
       last_code = code;
     }
-  token_end = buf_ptr;
-  if (buf_ptr >= buf_end)	/* check for input buffer empty */
+
+  if (buf_ptr >= buf_end)
     fill_buffer ();
   parser_state_tos->last_u_d = unary_delim;
 
+  if (parser_state_tos->last_token == cpp_operator)
+    return overloaded;
   return (code);
 }
 
-/* Add the given keyword to the keyword table, using val as the keyword type */
-addkey (key, val)
-     char *key;
-     enum rwcodes val;
+/* Add the given keyword to the keyword table, using val as
+   the keyword type */
+
+void
+addkey (char *key, enum rwcodes val)
 {
-  register struct templ *p;
+  struct templ *p;
 
   /* Check to see whether key is a reserved word or not. */
   if (is_reserved (key, strlen (key)) != 0)
