@@ -1,4 +1,6 @@
-/* Copyright (c) 1985 Sun Microsystems, Inc. Copyright (c) 1980 The Regents
+/* Copyright (c) 1994, Joseph Arceneaux.  All rights reserved
+
+   Copyright (c) 1985 Sun Microsystems, Inc. Copyright (c) 1980 The Regents
    of the University of California. Copyright (c) 1976 Board of Trustees of
    the University of Illinois. All rights reserved.
 
@@ -45,6 +47,7 @@ init_parser ()
 
   save_com.size = INITIAL_BUFFER_SIZE;
   save_com.end = save_com.ptr = xmalloc (save_com.size);
+  save_com.len = save_com.column = 0;
 
   di_stack_alloc = 2;
   di_stack = (int *) xmalloc (di_stack_alloc * sizeof (*di_stack));
@@ -60,7 +63,7 @@ reset_parser ()
   parser_state_tos->p_stack[0] = stmt;	/* this is the parser's stack */
   parser_state_tos->last_nl = true;	/* this is true if the last thing
 					   scanned was a newline */
-  parser_state_tos->last_token = semicolon;
+  parser_state_tos->last_token = start_token;
   parser_state_tos->box_com = false;
   parser_state_tos->cast_mask = 0;
   parser_state_tos->noncast_mask = 0;
@@ -92,8 +95,17 @@ reset_parser ()
   parser_state_tos->ind_stmt = false;
   parser_state_tos->procname = "\0";
   parser_state_tos->procname_end = "\0";
+  parser_state_tos->classname = "\0";
+  parser_state_tos->classname_end = "\0";
   parser_state_tos->pcase = false;
   parser_state_tos->dec_nest = 0;
+
+  parser_state_tos->il[0] = 0;
+  parser_state_tos->cstk[0] = 0;
+  parser_state_tos->paren_indents[0] = 0;
+
+  save_com.len = save_com.column = 0;
+
   di_stack[parser_state_tos->dec_nest] = 0;
 
   l_com = combuf + INITIAL_BUFFER_SIZE - 5;
@@ -136,6 +148,9 @@ inc_pstack ()
 	xrealloc ((char *) parser_state_tos->cstk,
 		  parser_state_tos->p_stack_size * sizeof (int));
     }
+
+  parser_state_tos->cstk[parser_state_tos->tos]
+    = parser_state_tos->cstk[parser_state_tos->tos - 1];
   return parser_state_tos->tos;
 }
 
@@ -187,11 +202,12 @@ debug_init ()
 
 #endif
 
-void
+enum exit_values
 parse (tk)
      enum codes tk;		/* the code for the construct scanned */
 {
   int i;
+  enum exit_values my_value = total_success;
 
 #ifdef DEBUG
   if (debug)
@@ -273,9 +289,6 @@ parse (tk)
 	      && parser_state_tos->block_init_level == 0
 	      && parser_state_tos->last_token != rparen
 	      && !btype_2)
-#if 0
-	      && !parser_state_tos->col_1)
-#endif
 	    {
 	      parser_state_tos->ind_level += brace_indent;
 	      parser_state_tos->i_l_follow += brace_indent;
@@ -297,15 +310,16 @@ parse (tk)
 		{
 		  parser_state_tos->ind_level += brace_indent;
 		  parser_state_tos->i_l_follow += brace_indent;
+#if 0
 		  if (parser_state_tos->p_stack[parser_state_tos->tos]
 		      == swstmt)
 		    case_ind += brace_indent;
+#endif
 		}
 
 	      if (parser_state_tos->p_stack[parser_state_tos->tos] == swstmt
-		  && case_indent
-		  >= ind_size)
-		parser_state_tos->ind_level -= ind_size;
+		  && case_indent > 0)
+		parser_state_tos->i_l_follow += case_indent;
 	      /* for a switch, brace should be two levels out from the code */
 	    }
 	}
@@ -344,7 +358,10 @@ parse (tk)
     case elselit:		/* scanned an else */
 
       if (parser_state_tos->p_stack[parser_state_tos->tos] != ifhead)
-	diag (1, "Unmatched 'else'", 0, 0);
+	{
+	  ERROR ("Unmatched 'else'", 0, 0);
+	  my_value = indent_error;
+	}
       else
 	{
 	  /* indentation for else should be same as for if */
@@ -369,21 +386,34 @@ parse (tk)
 	  parser_state_tos->p_stack[parser_state_tos->tos] = stmt;
 	}
       else
-	diag (1, "Stmt nesting error.", 0, 0);
+	{
+	  ERROR ("Stmt nesting error.", 0, 0);
+	  my_value = indent_error;
+	}
       break;
 
     case swstmt:		/* had switch (...) */
       inc_pstack ();
       parser_state_tos->p_stack[parser_state_tos->tos] = swstmt;
+#if 0
       parser_state_tos->cstk[parser_state_tos->tos] = case_ind;
-      /* save current case indent level */
-      parser_state_tos->il[parser_state_tos->tos] = parser_state_tos->i_l_follow;
-      case_ind = parser_state_tos->i_l_follow + case_indent;	/* cases should be one
-								   level down from
-								   switch */
-      /* statements should be two levels in */
-      parser_state_tos->i_l_follow += case_indent + ind_size;
+#endif
+      parser_state_tos->cstk[parser_state_tos->tos]
+	= case_indent + parser_state_tos->i_l_follow;
+      if (! btype_2)
+	parser_state_tos->cstk[parser_state_tos->tos] += brace_indent;
 
+      /* save current case indent level */
+      parser_state_tos->il[parser_state_tos->tos]
+	= parser_state_tos->i_l_follow;
+      /* case labels should be one level down from switch, plus
+	 `case_indent' if any.  Then, statements should be the `ind_size'
+	 further. */
+#if 0
+      case_ind = parser_state_tos->i_l_follow + case_indent;
+      parser_state_tos->i_l_follow += case_indent + ind_size;
+#endif
+      parser_state_tos->i_l_follow += ind_size;
       parser_state_tos->search_brace = btype_2;
       break;
 
@@ -403,12 +433,10 @@ parse (tk)
 	}
       break;
 
-    default:			/* this is an error */
-      diag (1, "Unknown code to parser", 0, 0);
-      return;
-
-
-    }				/* end of switch */
+      /* This is a fatal error which cases the program to exit. */
+    default:
+      fatal ("Unknown code to parser", 0, 0);
+    }
 
   reduce ();			/* see if any reduction can be done */
 
@@ -424,7 +452,7 @@ parse (tk)
     }
 #endif
 
-  return;
+  return total_success;
 }
 
 /* NAME: reduce
@@ -462,10 +490,11 @@ HISTORY: initial coding 	November 1976	D A Willcox of CAC
 /*----------------------------------------------*\
 |   REDUCTION PHASE				    |
 \*----------------------------------------------*/
+
+void
 reduce ()
 {
-
-  register int i;
+  int i;
 
   for (;;)
     {				/* keep looping until there is nothing left
@@ -506,7 +535,9 @@ reduce ()
 
 	    case swstmt:
 	      /* <switch> <stmt> */
+#if 0
 	      case_ind = parser_state_tos->cstk[parser_state_tos->tos - 1];
+#endif
 
 	    case decl:		/* finish of a declaration */
 	    case elsehead:
